//________________________________________________________________________________________________________
// OakLibTest_Hotplug.cpp
// 
// This is a very simple console application that demonstrates how to communicate with the Oak sensors,
// using the Oak library provided by Toradex.
// 
// It was tested in Visual Studio 2005 running on Windows XP
//
// June 29, 2007, Andy Kiser, Toradex AG
//________________________________________________________________________________________________________
#include <windows.h>
#include <stdio.h>
#include <tchar.h>
#include <math.h>	// pow function
#include "Oak.h"

#ifdef _UNICODE
#error ASCII mode required: check project properties -> Configuration Properties -> General -> Character Set
#endif

// __________________________ O a k _ P r i n t S e n s o r I n f o ______________________________________
// Print some information that is provided by the Oak Library during the initialization of the sensor
// (This routine contains no USB specific code, it just prints data)

void Oak_PrintSensorInfo(PtOakSensor Sensor)
{
	printf_s("===============================================\n");
	printf_s("%s (usr:%s)\n", Sensor->DeviceName, Sensor->UserDeviceName);
	printf_s("===============================================\n\n");
	printf_s("VID:%04X  PID:%04X  REV:%04X  SN:%s\n", Sensor->VID, Sensor->PID, Sensor->REV, Sensor->SN);
	printf_s("\nDevicePath:\n%s\n", Sensor->DevicePath);

	printf ("\n  %i Data Channels\n", Sensor->NumChannels);
	for (int ch=0; ch<Sensor->NumChannels; ch++)
	{
		printf_s("\n  ----------\n");
		printf_s("  Channel %i\n", ch);
		printf_s("  ----------\n");
		printf_s("  %s (usr:%s)\n", Sensor->Channel[ch].ChannelName, Sensor->Channel[ch].UserChannelName);
		if (Sensor->Channel[ch].IsSigned)
			printf_s("  %i Bit signed\n", Sensor->Channel[ch].BitSize);
		else
			printf_s("  %i Bit unsigned\n", Sensor->Channel[ch].BitSize);
		printf("  Resolution: 1e%i %s\n", Sensor->Channel[ch].UnitExponent, Sensor->Channel[ch].UnitStr);
	}
}

// __________________________ O a k _ P r i n t S e n s o r V a l u e s __________________________________
// Print measured values
// This function shows how to use the information provided in the tOakSensor data structure to
// correctly display the sensor's measured values

void Oak_PrintSensorValues(PtOakSensor Sensor, BYTE *InReport)
{
	double	Val;
	BYTE	RptIdx=1;
	char	NofDigits;

	printf_s("\n  Current sensor values are:\n");
	printf_s("  --------------------------\n");
	// loop through every data channel
	for (BYTE i=0; i<Sensor->NumChannels; i++)
	{
		// evaluate the numeric value of the channel
		// distinguish between 8/16 Bit values and signed/unsigned values
		switch (Sensor->Channel[i].BitSize)
		{
			case 8:	
				if (Sensor->Channel[i].IsSigned)		// 8bit signed
					Val = (char)InReport[RptIdx];
				else
					Val = (BYTE)InReport[RptIdx];	// 8bit unsigned
				RptIdx++;
				break;

			case 16:
				if (Sensor->Channel[i].IsSigned)		
					Val = (int )( InReport[RptIdx] + 256*InReport[RptIdx+1] );	// 16bit signed
				else							
					Val = (WORD)( InReport[RptIdx] + 256*InReport[RptIdx+1] );	// 16bit unsigned
				RptIdx += 2;
				break;
		}

		// Multiply with the exponent that is also provided in the standard USB/HID descriptor
		Val *= pow(10.0f, Sensor->Channel[i].UnitExponent);
	
		// use the same exponent information to decide how many fractional digits we will print
		NofDigits = -Sensor->Channel[i].UnitExponent;
		if (NofDigits <0) 
			NofDigits = 0;

		// transform to string
		printf_s("  %.*f [%s]\r\n", NofDigits, Val, Sensor->Channel[i].UnitStr);
	}
}

// __________________________ O a k _ O n C o n n e c t __________________________________________________
// This is a Callback function that gets called 
// - each time a sensor matching specified criteria is plugged in
// - each time the application calls Oak_NotifyConnectedDevices(), and a sensor matching specified 
//   criteria is connected to the host
// Be aware that this function may get called more than once. For example
// - if more than one sensor matches the specified criteria
// - if the application calls Oak_NotifyConnectedDevices() more than once.

void __stdcall Oak_OnConnect (PtOakSensor Sensor, LPARAM Param)
{
	BYTE InReport[33];

	printf_s("\n\n--> Connected: <--\n");
	Oak_PrintSensorInfo(Sensor);

	// Read current sensor values
	// This is one of two similar commands to get the current sensor values (the preferred one)
	// Param 1 Device path at it is provided in the PtOakSensor Structure
	// Param 2 Buffer to store the Input report. If you're not sure about its size, reserve 33 Bytes to use it with 
	//         any Oak sensor device.
	// Param 3 is the report size. It has to match exactly, otherwise we do not get anything. Remember, by now
	//         every channel data is 16 bits (2 bytes); in addition there is one preceding byte that contains the 
	//         report number (this is always 0)
	// Param 4 is a timeout value
	//         In contrast to the method above, this function reads every value reported by the sensor device exactly once.
	//         If there is no updated value available within 1933ms in this example, the function returns indicating a failure.
	bool fSuccess = Oak_GetInReport(Sensor->DevicePath, InReport, 1+2*Sensor->NumChannels, 1933);
	if (!fSuccess)
		printf_s("/* ERROR: ReadFile failed */\n");
	else
		Oak_PrintSensorValues(Sensor, InReport);

}
// __________________________ O a k _ O n D i s c o n n e c t ____________________________________________
// This is a Callback function that gets called each time a sensor matching specified 
// criteria is disconnected

void __stdcall Oak_OnDisconnect (PtOakSensor Sensor, LPARAM Param)
{
	printf_s("\n\n--> Disconnected: <--\n");
	Oak_PrintSensorInfo(Sensor);
}

// __________________________ _t m a i n _________________________________________________________________
// this is already the main function.
// As you see, I tried to keep the complexity of the application low, so you can easily find what this 
// application wants to show: communication with the Oak sensor.
int _tmain(int argc, _TCHAR* argv[])
{
	// Define the criteria for the sensor(s) to look for, and the callback functions 
	// that will be called on connecting / disconnecting a matching sensor.
	Oak_Register_Callback(Oak_OnConnect,	// function that gets called when a matching sensor appears
		                  Oak_OnDisconnect, // function that gets called when a matching sensor disappears
						  0x0000,           // search for ANY PID (product ID)
						  0x0000,			// search for ANY REV (product revision)
						  NULL,				// search for ANY serial number
						  NULL,				// search for ANY device name
						  NULL,				// search for ANY channel name
						  NULL,				// search for ANY user defined device name
						  NULL,				// search for ANY user defined channel name
						  0xABCD);          // This value will be passed to the Callback functions
	// Activate Hotplug functionality
	Oak_Hotplug_Enable();

	// Simulate Connecting. This is necessary to be able to detect sensors that are currently
	// already connected.
	// Now "Oak_OnConnect" is called once for every connected matching sensor
	Oak_NotifyConnectedDevices();


	// Just wait for the <ENTER> key, before closing the window
	char ch = getchar();
	return 0;
}

